# Current version of OVS that this package requires
%define ovs_version 3.1

# See below: ovs_version_short will be first two sections
#            separated by '.'

# Comma-separated (no spaces) e.g. 2.10,2.9 ... of prior fast-datapath
# openvswitch and ovn packages we need to obsolete
%define obsolete_ovs_versions 2.15,2.17,3.0

# Same as above, but enable ovs/ovn to be separate
%define ovn_version 23.09

# See below: ovn_version_short will be first (up to) two sections
#            separated by '.'

# Use trimmed versions here (e.g. 22.03.0 -> 22.03)
%define obsolete_ovn_versions 2.15,21.09,21.12,22.03,22.06,22.12

%if 0%{?rhel} > 7
%define pyprefix python3
%else
%define pyprefix python
%endif

# 05-27-2022
# OVN versioning scheme is: packageMAJ.MIN-MAJ.MIN.PATCHLEVEL (or
#                           sometimes no PATCHLEVEL)
# OVS versioning scheme is: packageMAJ.MIN-MAJ.MIN
#
# Whenever we generate provides/obsoletes, we need to require the
# appropriate MAJ.MIN but omit anything after that, so we trim
# to 2 version slots when generating our requires/obsoletes.
#
# Lua macro to create a bunch of Obsoletes by splitting up the above
# definition and substituting where there's an asterisk
%{lua:
function simple_split(s, sep)
  local ret = {}
  for blob in string.gmatch(s, "[^"..sep.."]+") do
    table.insert(ret, blob)
  end
  return ret
end

function simple_join(s, sep, n)
  n = n or 0
  if n == 0 then
    return table.concat(s, sep)
  else
    return table.concat(s, sep, 1, n)
  end
end

function trim_version(version_str)
    return simple_join(simple_split(version_str, "."), ".", 2)
end

function set_short_version(ver)
    rpm.define(ver.."_short "..trim_version(rpm.expand("%"..ver)).."\n")
end

function rhosp_obsoletes(package, ver, obsoletes)
  local s
  local pkg
  pkg = string.gsub(package, "*", "")
  print("Obsoletes: "..pkg.." < "..ver.."\n")
  for s in string.gmatch(obsoletes, "[^,]+") do
    pkg = string.gsub(package, "*", s)
    print("Obsoletes: "..pkg.." < "..ver.."\n")
  end
end

set_short_version("ovs_version")
set_short_version("ovn_version")

function ovs_obsoletes(package)
  rhosp_obsoletes(package, rpm.expand("%ovs_version"), rpm.expand("%obsolete_ovs_versions"))
end

function ovn_obsoletes(package)
  rhosp_obsoletes(package, rpm.expand("%ovn_version"), rpm.expand("%obsolete_ovn_versions"))
end}

######## OPENVSWITCH PACKAGING ########

Name:           rhosp-openvswitch
Version:        %{ovs_version}
Release:        4%{?dist}
Summary:        Wrapper rpm to allow installing OVS with new versioning schemes

Group:          System Environment/Daemons
License:        Public domain
URL:            http://www.openvswitch.org
BuildArch:      noarch

Requires:       openvswitch%{ovs_version_short}
Requires:       rhosp-network-scripts-openvswitch = %{?epoch:%{epoch}:}%{version}-%{release}
Provides:       openvswitch = %{ovs_version}
%{lua:ovs_obsoletes("openvswitch*")}

%description
Wrapper rpm for the base openvswitch package

%package -n %{pyprefix}-rhosp-openvswitch
Summary:    wrapper for python-openvswitch rpm
License:    Public domain
Requires:   %{name} = %{?epoch:%{epoch}:}%{version}-%{release}
Requires:   %{pyprefix}-openvswitch%{ovs_version_short}
Provides:   %{pyprefix}-openvswitch = %{ovs_version}
%if 0%{?rhel} > 7
%{lua:ovs_obsoletes("python3-openvswitch*")}
# Just in case / being pedantic ...
%{lua:ovs_obsoletes("python2-openvswitch*")}
%endif
# RHEL7 builds didn't have python2-openvswitch
%{lua:ovs_obsoletes("python-openvswitch*")}

%description -n %{pyprefix}-rhosp-openvswitch
Wrapper rpm for the base %{pyprefix}-openvswitch package

%package devel
Summary:    wrapper for openvswitch-devel rpm
License:    Public domain
Requires:   %{name} = %{?epoch:%{epoch}:}%{version}-%{release}
Requires:   openvswitch%{ovs_version_short}-devel
Provides:   openvswitch-devel = %{ovs_version}
%{lua:ovs_obsoletes("openvswitch*-devel")}

%description devel
Wrapper rpm for the base openvswitch-devel package

%package test
Summary:    wrapper for openvswitch-test rpm
License:    Public domain
Requires:   %{pyprefix}-rhosp-openvswitch = %{?epoch:%{epoch}:}%{version}-%{release}
Requires:   openvswitch%{ovs_version_short}-test
Provides:   openvswitch-test = %{ovs_version}
%{lua:ovs_obsoletes("openvswitch*-test")}

%description test
Wrapper rpm for the base openvswitch-test package

%package -n rhosp-network-scripts-openvswitch
Summary:    wrapper for network-scripts-openvswitch rpm
License:    Public domain
Requires:   network-scripts-openvswitch%{ovs_version_short}
Provides:   network-scripts-openvswitch = %{ovs_version}
%{lua:ovs_obsoletes("network-scripts-openvswitch*")}

%description -n rhosp-network-scripts-openvswitch
Wrapper rpm for the base network-scripts-openvswitch package

######## OVN PACKAGING ########

%package -n rhosp-ovn
Version:    %{ovn_version}
Summary:    wrapper for ovn rpm
License:    Public domain
Requires:   openvswitch
Requires:   ovn%{ovn_version_short}
Provides:   ovn = %{ovn_version}
Provides:   openvswitch-ovn-common = %{ovn_version}
Provides:   %{name}-ovn-common = %{ovn_version}
Obsoletes:  %{name}-ovn-common < %{?epoch:%{epoch}:}%{ovn_version}-%{release}
Provides:   rhosp-ovn-common = %{ovn_version}
Obsoletes:  rhosp-ovn-common < %{?epoch:%{epoch}:}%{ovn_version}-%{release}

# OVN 2021 builds were versioned in a non-standard way
Obsoletes:  ovn-2021 < 22
# OVN packaging should do this, but doesn't?
# Obsoletes: openvswitch-ovn-common < ...
%{lua:ovn_obsoletes("ovn*")}

%description -n rhosp-ovn
Wrapper rpm for the base ovn package

%package -n rhosp-ovn-central
Version:    %{ovn_version}
Summary:    wrapper for openvswitch-ovn-central rpm
License:    Public domain
Requires:   rhosp-ovn = %{?epoch:%{epoch}:}%{version}-%{release}
Requires:   ovn%{ovn_version}-central
Provides:   openvswitch-ovn-central = %{ovn_version}
Obsoletes:  openvswitch-ovn-central < %{ovn_version}
Provides:   ovn-central = %{ovn_version}
Obsoletes:  ovn-central < %{ovn_version}
# OVN 2021 builds were versioned in a non-standard way
Obsoletes:  ovn-2021-central < %{ovn_version}
Provides:   %{name}-ovn-central = %{ovn_version}
Obsoletes:  %{name}-ovn-central < %{?epoch:%{epoch}:}%{ovn_version}-%{release}
%{lua:ovn_obsoletes("openvswitch*-ovn-central")}
%{lua:ovn_obsoletes("ovn*-central")}

%description -n rhosp-ovn-central
Wrapper rpm for the base openvswitch-ovn-central package

%package -n rhosp-ovn-host
Version:    %{ovn_version}
Summary:    wrapper for openvswitch-ovn-host rpm
License:    Public domain
Requires:   rhosp-ovn = %{?epoch:%{epoch}:}%{version}-%{release}
Requires:   ovn%{ovn_version}-host
Provides:   openvswitch-ovn-host = %{ovn_version}
Obsoletes:  openvswitch-ovn-host < %{ovn_version}
Provides:   ovn-host = %{ovn_version}
Obsoletes:  ovn-host < %{ovn_version}
# OVN 2021 builds were versioned in a non-standard way
Obsoletes:  ovn-2021-host < %{ovn_version}
Provides:   %{name}-ovn-host = %{ovn_version}
Obsoletes:  %{name}-ovn-host < %{?epoch:%{epoch}:}%{ovn_version}-%{release}
%{lua:ovn_obsoletes("openvswitch*-ovn-host")}
%{lua:ovn_obsoletes("ovn*-host")}

%description -n rhosp-ovn-host
Wrapper rpm for the base openvswitch-ovn-host package

%package -n rhosp-ovn-vtep
Version:    %{ovn_version}
Summary:    wrapper for openvswitch-ovn-vtep rpm
License:    Public domain
Requires:   rhosp-ovn = %{?epoch:%{epoch}:}%{ovn_version}-%{release}
Requires:   ovn%{ovn_version}-vtep
Provides:   openvswitch-ovn-vtep = %{ovn_version}
Obsoletes:  openvswitch-ovn-vtep < %{ovn_version}
Provides:   ovn-vtep = %{ovn_version}
Obsoletes:  ovn-vtep < %{ovn_version}
# OVN 2021 builds were versioned in a non-standard way
Obsoletes:  ovn-2021-vtep < %{ovn_version}
Provides:   %{name}-ovn-vtep = %{ovn_version}
Obsoletes:  %{name}-ovn-vtep < %{?epoch:%{epoch}:}%{ovn_version}-%{release}
%{lua:ovn_obsoletes("openvswitch*-ovn-vtep")}
%{lua:ovn_obsoletes("ovn*-vtep")}

%description -n rhosp-ovn-vtep
Wrapper rpm for the base ovn-vtep package

%setup -q

%build

%files
%files -n %{pyprefix}-rhosp-openvswitch
%files devel
%files test
%files -n rhosp-ovn
%files -n rhosp-ovn-central
%files -n rhosp-ovn-host
%files -n rhosp-ovn-vtep
%files -n rhosp-network-scripts-openvswitch

%changelog
* Fri Mar 15 2024 Miguel Garcia <migarcia@redhat.com> - 3.1-4
- Update OVN to 23.09

- Fri Mar 17 2023 Miguel Garcia <migarcia@redhat.com> 3.1-2
- Bump release

* Fri Mar 17 2023 Miguel Garcia <migarcia@redhat.com> 3.1-1
- Update to use OVS 3.1

* Mon Jan 16 2023 Miguel Garcia <migarcia@redhat.com> 3.0-1
- Update to use OVS 3.0 and OVN 22.12

* Thu Oct 06 2022 Miguel Garcia <migarcia@redhat.com> 2.17-6
- Update to use OVN 22.06

* Tue May 31 2022 Lon Hohberger <lhh@redhat.com> 2.17-5
- Fix up Obsoletes for ovn-2021-* (rhbz#2091952)

* Fri May 27 2022 Lon Hohberger <lhh@redhat.com> 2.17-4
- Allow 3+ version tuples (maj/min/patch)
- Obsolete ovn-2021 < 22 to avoid micro version bumps
- Update to use ovn22.03 (rhbz#2076719)

* Wed Apr 13 2022 Jon Schlueter <jschluet@redhat.com> 2.17-1
- update to use 2.17 (rhbz#2076719)

* Tue Feb 08 2022 Jon Schlueter <jschluet@redhat.com> 2.15-6
- remove guard on rhel version guard and always Require rhosp-network-scripts-openvswitch rhbz#2052057

* Fri May 21 2021 Mike Burns <mburns@redhat.com> - 2.15-4
- fix requires for OVN to handle new naming

* Mon May 10 2021 Shreshtha Joshi <shrjoshi@redhat.com> - 2.15-3
- Update for network-script

* Mon May 03 2021 Shreshtha Joshi <shrjoshi@redhat.com> - 2.15-2
- Update OVN to 2021

* Wed Sep 9 2020 Lon Hohberger <lon@redhat.com> 2.13-9
- Fix a few requires/provides in OVN packages

* Mon Apr 20 2020 Lon Hohberger <lon@redhat.com> 2.13-8
- Allow OVS and OVN versions to diverge
- Fix requirements / obsoletes / provides preventing certain upgrades from 2.11-0.6

* Thu Mar 19 2020 Lon Hohberger <lon@redhat.com> 2.13-3
- Fix OVN wrappers

* Wed Mar 18 2020 Lon Hohberger <lon@redhat.com> 2.13-2
- Mirror openvswitch2.13 internal dependencies in wrappers

* Mon Mar 16 2020 Thierry Vignaud <tvignaud@redhat.com> 2.13-1
- rebase to OVS 2.13

* Tue Jul 30 2019 Lon Hohberger <lon@redhat.com> - 2.11-0.6
- Use Obsoletes: lua macro so we can add/remove versions to Obsoletes
  without rewriting the spec file each time

* Tue Jun 11 2019 Thierry Vignaud <tvignaud@redhat.com> 2.11-0.1.fc29
- Update for OVS 2.11

* Wed Jul 11 2018 Mike Burns <mburns@redhat.com> - 2.10-0.1
- initial spec for RHOSP openvswitch wrapper RPMs
